%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
%% @doc NOTE: The low-level debugging support is EXPERIMENTAL!
%%
%% <p>TODO: There is also a pseudo-BIF named erts_debug:apply/4.</p>
%% <p>TODO: Separate erlang-debug and erts-debug?</p>

-module(erl.system.debug).

-export([breakpoint/2, df/1, df/2, df/3, disassemble/1, display/1,
	 display_nl/0, display_string/1, flat_size/1, make_fun/1,
	 process_display/2, resume_process/1, same/2, seq_trace/2,
	 seq_trace_info/1, seq_trace_print/1, seq_trace_print/2, size/1,
	 suspend_process/1, trace/3, trace_info/2, trace_pattern/2,
	 trace_pattern/3]).


%% @doc Returns the size of Term in actual heap words. Shared subterms
%% are counted once.
size(Term) ->
    erts_debug:size(Term).


%% @doc Disassemble Beam code to file.
df(Mod) ->
    erts_debug:df(Mod).

%% @doc Disassemble Beam code to file.
df(Mod, Func) ->
    erts_debug:df(Mod, Func).

%% @doc Disassemble Beam code to file.
df(Mod, Func, Arity) ->
    erts_debug:df(Mod, Func, Arity).


%% @doc <em>Experimental</em>.
breakpoint(MFA, Bool) ->
    erl.system.debug:breakpoint(MFA, Bool).    % Built-in function


%% @doc <em>Experimental</em>.
disassemble(Addr) ->
    erl.system.debug:disassemble(Addr).    % Built-in function


%% @doc See {@link /kernel/erlang:display/1}.
display(Term) ->
    erl.system.debug:display(Term).    % Built-in function


%% @doc Undocumented. Avoid.
display_nl() ->
    erl.system.debug:display_nl().    % Built-in function


%% @doc Undocumented. Avoid.
display_string(String) ->
    erl.system.debug:display_string(String).    % Built-in function


%% @doc <em>Experimental</em>.
flat_size(Term) -> erl.system.debug:flat_size(Term).    % Built-in function


%% @doc Undocumented. Do not use.
make_fun(Tuple) ->
    erl.system.debug:make_fun(Tuple).    % Built-in function


%% @doc See {@link /kernel/erlang:process_display/2}.
process_display(Pid, Type) ->
    erl.system.debug:process_display(Pid, Type).    % Built-in function


%% @doc See {@link /kernel/erlang:resume_process/1}.
resume_process(Pid) ->
    erl.system.debug:resume_process(Pid).    % Built-in function


%% @doc <em>Experimental</em>.
same(Term1, Term2) ->
    erl.system.debug:same(Term1, Term2).    % Built-in function


%% @doc Undocumented. Do not use.
seq_trace(X,Y) ->
    erl.system.debug:seq_trace(X,Y).    % Built-in function

%% @doc Undocumented. Do not use.
seq_trace_info(X) ->
    erl.system.debug:seq_trace_info(X).    % Built-in function

%% @doc Undocumented. Do not use.
seq_trace_print(X) ->
    erl.system.debug:seq_trace_print(X).    % Built-in function

%% @doc Undocumented. Do not use.
seq_trace_print(X,Y) ->
    erl.system.debug:seq_trace_print(X,Y).    % Built-in function


%% @doc See {@link /kernel/erlang:suspend_process/1}.
suspend_process(Pid) ->
    erl.system.debug:suspend_process(Pid).    % Built-in function


%% @doc See {@link /kernel/erlang:trace/3}.
trace(PidSpec, How, Flaglist) ->
    erl.system.debug:trace(PidSpec, How, Flaglist).    % Built-in function


%% @doc See {@link /kernel/erlang:trace_info/2}.
trace_info(PidOrFunc, Item) ->
    erl.system.debug:trace_info(PidOrFunc, Item).    % Built-in function


%% @doc See {@link /kernel/erlang:trace_pattern/2}.
trace_pattern(MFA, MatchSpec) ->
    erl.system.debug:trace_pattern(MFA, MatchSpec).    % Built-in function


%% @doc See {@link /kernel/erlang:trace_pattern/3}.
trace_pattern(MFA, MatchSpec, FlagList) ->
    erl.system.debug:trace_pattern(MFA, MatchSpec, FlagList).    % Built-in function
