%%% -*- erlang-indent-level: 2 -*-
%%% $Id: hipe_rtl_gctests.erl,v 1.14 2004/03/03 08:24:31 pergu Exp $
%%%
%%% Expand virtual gc tests to actual code.

-module(hipe_rtl_gctests).
-export([expand/1]).

expand(CFG) ->
   Labels = hipe_rtl_cfg:labels(CFG),
   expand_gctests(Labels, CFG).

expand_gctests([], CFG) ->
   CFG;
expand_gctests([L|Ls], CFG) ->
   BB = hipe_rtl_cfg:bb(CFG, L),
   Code = hipe_bb:code(BB),
   {NewCode, CFG0} = expand_gc(Code, CFG),
   CFG1 = hipe_rtl_cfg:bb_add(CFG0, L, hipe_bb:code_update(BB, NewCode)),
   expand_gctests(Ls, CFG1).

expand_gc([], CFG) ->
  {[], CFG};
expand_gc([I|Is], CFG) ->
  {CodeRest, CFG0} = expand_gc(Is, CFG),
  case hipe_rtl:type(I) of
    gctest ->  %% Two new blocks: The rest of this one, and one for the GC.
      %% Block One
      ContLabel = hipe_rtl:label_name(hipe_rtl:mk_new_label()),
      CFG1 = hipe_rtl_cfg:bb_add(CFG0, ContLabel, hipe_bb:mk_bb(CodeRest)),
      %% Block Two
      %% Note: the pointer arithmetic is done in bytes,
      %% but the gc parameter is in words (tagged as fixnum).
      HeapNeed = hipe_rtl:gctest_words(I),
      Tmp = hipe_rtl:mk_new_reg(),
      {GetHPInsn, HP, _PutHPInsn} = hipe_rtl_arch:heap_pointer(),
      {GetHLIMITInsn, H_LIMIT} = hipe_rtl_arch:heap_limit(),
      GCLabel = hipe_rtl:label_name(hipe_rtl:mk_new_label()),	 
   
      case hipe_rtl:is_reg(HeapNeed) of	
	true -> % Creates a gctest for a number of words known at runtime
	  HPAmount = hipe_rtl:mk_new_reg(),
	  GCAmount = hipe_rtl:mk_new_var(),
	  Code = [hipe_rtl:mk_alu(HPAmount, HeapNeed, sll, 
				  hipe_rtl:mk_imm(hipe_rtl_arch:log2_word_size())),
		  hipe_rtl:mk_alu(Tmp, H_LIMIT, 'sub', HP)|
		  hipe_tagscheme:tag_fixnum(GCAmount, HeapNeed)] ++
		  [hipe_rtl:mk_branch(Tmp, 'lt', HPAmount, GCLabel, 
				      ContLabel, 0.01)];
	false -> % Creates a gctest for a fixed number of words
	  WordsNeeded = hipe_rtl:imm_value(HeapNeed),
	  GCAmount = hipe_rtl:mk_imm(hipe_tagscheme:mk_fixnum(WordsNeeded)),
	  HPAmount = hipe_rtl:mk_imm(WordsNeeded*hipe_rtl_arch:word_size()),
	  %% Le grande finale
	  %% Do the GC overflow test. In the old version,
	  %%	if( (Tmp = HP+HPAmount) >= H_LIMIT ) goto GCLabel,
	  %% Tmp could wrap around. The new version below should be safe.
	  Code = [hipe_rtl:mk_alu(Tmp, H_LIMIT, 'sub', HP),
		  hipe_rtl:mk_branch(Tmp, 'lt', HPAmount, GCLabel, ContLabel, 0.01)]
      end,
      GCCode = [hipe_rtl:mk_call([], gc_1, [GCAmount], ContLabel, [])],
      CFG2 = hipe_rtl_cfg:bb_add(CFG1, GCLabel, hipe_bb:mk_bb(GCCode)),

      {[GetHPInsn, GetHLIMITInsn | Code], CFG2};
    _ ->
      {[I|CodeRest], CFG0}
  end.
